@@ -31,10 +31,8 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
 from gettext import gettext as _
 import apt
 import logging
-import operator
 import itertools
 import platform
-import subprocess
 import os
 import random
 import glob
@@ -50,6 +48,9 @@ class UpdateItem():
         self.name = name
         self.pkg = pkg
 
+    def is_selected(self):
+        return self.pkg.marked_install or self.pkg.marked_upgrade
+
 
 class UpdateGroup(UpdateItem):
     def __init__(self, pkg, name, icon):
@@ -74,40 +75,42 @@ class UpdateGroup(UpdateItem):
     def contains(self, item):
         return item in self._items
 
-    def _is_dependency_helper(self, cache, pkg, dep, seen=set()):
+    def _is_dependency_helper(self, cache, pkg, dep, seen, eventloop_callback):
         if pkg is None or pkg.candidate is None or pkg in seen:
             return False
         elif pkg.name == dep.name:
             return True
+        if eventloop_callback is not None:
+            eventloop_callback()
         seen.add(pkg)
         candidate = pkg.candidate
         dependencies = candidate.get_dependencies('Depends', 'Recommends')
         for dependency_pkg in itertools.chain.from_iterable(dependencies):
-            if dependency_pkg.name in cache and \
-               self._is_dependency_helper(cache, cache[dependency_pkg.name],
-                                          dep, seen):
+            if (dependency_pkg.name in cache and
+                self._is_dependency_helper(
+                    cache, cache[dependency_pkg.name], dep, seen,
+                    eventloop_callback)):
                 return True
         return False
 
-    def is_dependency(self, cache, maybe_dep):
+    def is_dependency(self, cache, maybe_dep, eventloop_callback):
         # This is a recursive dependency check.  TODO: We do this many times
         # when grouping packages, and it could be made more efficient.
         seen = set()
         for item in self._items:
             if self._is_dependency_helper(cache, item.pkg, maybe_dep,
-                                          seen=seen):
+                                          seen, eventloop_callback):
                 return True
         return False
 
     def packages_are_selected(self):
         for item in self.items:
-            if item.pkg.marked_install or item.pkg.marked_upgrade:
+            if item.is_selected():
                 return True
         return False
 
     def selection_is_inconsistent(self):
-        pkgs_installing = [item for item in self.items
-            if item.pkg.marked_install or item.pkg.marked_upgrade]
+        pkgs_installing = [item for item in self.items if item.is_selected()]
         return (len(pkgs_installing) > 0 and
                 len(pkgs_installing) < len(self.items))
 
@@ -184,7 +187,7 @@ class UpdateList():
         if 'XDG_DATA_DIRS' in os.environ and os.environ['XDG_DATA_DIRS']:
             data_dirs = os.environ['XDG_DATA_DIRS']
         else:
-            data_dirs= '/usr/local/share/:/usr/share/'
+            data_dirs = '/usr/local/share/:/usr/share/'
         self.application_dirs = [os.path.join(base, 'applications')
                                  for base in data_dirs.split(':')]
 
@@ -235,7 +238,7 @@ class UpdateList():
                 application.set_desktop_env(self.current_desktop)
             except Exception as e:
                 print("Error loading .desktop file %s: %s" %
-                      (installed_file, e))
+                      (desktop_file, e))
                 continue
             score = self._rate_application_for_package(application, pkg)
             if score > 0:
@@ -321,7 +324,7 @@ class UpdateList():
                 'linux-image-virtual', 'linux-virtual',
                 'linux-image-extra-virtual']
 
-    def _make_groups(self, cache, pkgs):
+    def _make_groups(self, cache, pkgs, eventloop_callback):
         pkgs_by_source = {}
         ungrouped_pkgs = []
         app_groups = []
@@ -345,7 +348,7 @@ class UpdateList():
         for pkg in list(ungrouped_pkgs):
             dep_groups = []
             for group in app_groups:
-                if group.is_dependency(cache, pkg):
+                if group.is_dependency(cache, pkg, eventloop_callback):
                     dep_groups.append(group)
                     if len(dep_groups) > 1:
                         break
@@ -363,7 +366,8 @@ class UpdateList():
             if pkg in cache:
                 meta_group.add(cache[pkg])
         for pkg in ungrouped_pkgs:
-            if meta_group.contains(pkg) or meta_group.is_dependency(cache, pkg):
+            if (meta_group.contains(pkg) or
+                    meta_group.is_dependency(cache, pkg, eventloop_callback)):
                 if system_group is None:
                     system_group = UpdateSystemGroup(cache)
                 system_group.add(pkg)
@@ -377,7 +381,7 @@ class UpdateList():
 
         return app_groups + pkg_groups
 
-    def update(self, cache):
+    def update(self, cache, eventloop_callback=None):
         self.held_back = []
 
         # do the upgrade
@@ -412,5 +416,7 @@ class UpdateList():
                                           pkg.marked_install):
                 self.held_back.append(pkg.name)
 
-        self.update_groups = self._make_groups(cache, upgrade_pkgs)
-        self.security_groups = self._make_groups(cache, security_pkgs)
+        self.update_groups = self._make_groups(cache, upgrade_pkgs,
+                                               eventloop_callback)
+        self.security_groups = self._make_groups(cache, security_pkgs,
+                                                 eventloop_callback)
