@@ -35,8 +35,7 @@ import dbus
 import os
 import subprocess
 import sys
-import time
-from .SimpleGtk3builderApp import SimpleGtkbuilderApp
+from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
 
 from gettext import gettext as _
 
@@ -90,13 +89,7 @@ class Dialog(SimpleGtkbuilderApp):
                #        this back
                #"--toplevel", "%s" % self.window_main.get_window().get_xid()
                ]
-        self.window_main.set_sensitive(False)
-        p = subprocess.Popen(cmd)
-        while p.poll() is None:
-            while Gtk.events_pending():
-                Gtk.main_iteration()
-            time.sleep(0.05)
-        self.window_main.set_sensitive(True)
+        subprocess.Popen(cmd)
 
     def set_header(self, label):
         self.label_header.set_label(label)
@@ -119,9 +112,12 @@ class StoppedUpdatesDialog(Dialog):
 
 
 class NoUpdatesDialog(Dialog):
-    def __init__(self, window_main):
+    def __init__(self, window_main, error_occurred=False):
         Dialog.__init__(self, window_main)
-        self.set_header(_("The software on this computer is up to date."))
+        if error_occurred:
+            self.set_header(_("No software updates are available."))
+        else:
+            self.set_header(_("The software on this computer is up to date."))
         self.add_settings_button()
         self.focus_button = self.add_button(Gtk.STOCK_OK, self.close)
 
@@ -141,9 +137,18 @@ class DistUpgradeDialog(Dialog):
         self.focus_button = self.add_button(Gtk.STOCK_OK, self.close)
 
     def upgrade(self):
+        # Pass on several arguments
+        extra_args = ""
+        if self.window_main and self.window_main.options:
+            if self.window_main.options.devel_release:
+                extra_args = extra_args + " -d"
+            if self.window_main.options.use_proposed:
+                extra_args = extra_args + " -p"
+            if self.window_main.options.sandbox:
+                extra_args = extra_args + " -s"
         os.execl("/bin/sh", "/bin/sh", "-c",
                  "/usr/bin/pkexec /usr/bin/do-release-upgrade "
-                 "--frontend=DistUpgradeViewGtk3")
+                 "--frontend=DistUpgradeViewGtk3%s" % extra_args)
 
 
 class UnsupportedDialog(DistUpgradeDialog):
@@ -205,15 +210,23 @@ class ErrorDialog(Dialog):
     def main(self):
         Dialog.main(self)
         # The label likes to start selecting everything (b/c it got focus
-        # before
-        # we switched to our default button).
+        # before we switched to our default button).
         self.label_desc.select_region(0, 0)
-        # Since errors usually are outside the normal flow, we'll guarantee
-        # that
-        # we don't continue with normal code flow by running our own loop here.
-        # We won't screw anything up because the only thing this dialog will do
-        # is exit.
-        Gtk.main()
+
+
+class UpdateErrorDialog(ErrorDialog):
+    def __init__(self, window_main, header, desc=None):
+        ErrorDialog.__init__(self, window_main, header, desc)
+        # Get rid of normal error dialog button before adding our own
+        self.focus_button.destroy()
+        self.add_button(_("_Try Again"), self.update)
+        self.focus_button = self.add_button(Gtk.STOCK_OK, self.available)
+
+    def update(self):
+        self.window_main.start_update()
+
+    def available(self):
+        self.window_main.start_available(error_occurred=True)
 
 
 class NeedRestartDialog(Dialog):
