@@ -28,8 +28,6 @@ class PhasedTestCase(unittest.TestCase):
                                     "aptroot-update-list-test")
         self.cache = MyCache(apt.progress.base.OpProgress(),
                              rootdir=self.aptroot)
-        self.cache._listsLock = 0
-        self.cache.update()
         self.cache.open()
         self.updates_list = UpdateList.UpdateList(parent=None)
 
@@ -86,6 +84,7 @@ class PhasedTestCase(unittest.TestCase):
             self.updates_list.update(self.cache)
             self.assertUpdatesListLen(2)
 
+
 class GroupingTestCase(unittest.TestCase):
     # installed_files does not respect aptroot, so we have to patch it
     @patch('apt.package.Package.installed_files', new_callable=PropertyMock)
@@ -100,6 +99,7 @@ class GroupingTestCase(unittest.TestCase):
                                     "aptroot-grouping-test")
         self.cache = MyCache(apt.progress.base.OpProgress(),
                              rootdir=self.aptroot)
+        self.cache.open()
         mock_installed.__get__ = self.fake_installed_files
         mock_desktop.side_effect = self.fake_desktop
         self.updates_list = UpdateList.UpdateList(parent=None, dist='lucid')
